/*
        This file is part of PuDaC.

        PuDaC is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 3 of the License, or
        (at your option) any later version.

        PuDaC is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with PuDaC  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef __PUDAC_H__
#define __PUDAC_H__

#include <search.h>
#include <stdlib.h>

#define error(format, args...)  \
 fprintf (stderr, format "\n" , ## args)

/* val_symbol, linked should be NULL when not used --
 i.e. type==_float: val_float=something, val_symbol=0
      type==_symbol: val_float=(0?), val_symbol=ptr
      type==list: val_float & val_symbol - as appropriate
                 val_linked=ptr to next */

#define PDC_CALLBACK 0x4342
#define PDC_SENT     0x534E
#define PDC_DSP_DAC -0x4443

#define DSP_MARK_VALID(s) { \
  char * t = &(s->is_dsp);  \
  *t = 1;                   \
}

struct PDdata {
  enum PDtype { PDC_TYPE_FLOAT, 
		PDC_TYPE_BANG, 
		PDC_TYPE_SYMBOL, 
		PDC_DSP_REQUEST,
		pdc_last_type } type;
  float val_float;
  char * val_symbol;
  int sample_count;
  char is_dsp;
  float * dsp_stream;
  struct PDdata * linked;
};

/* Code Doesn't Belong Here, but it doesn't belong in another module, either, so... */

typedef void (*objectfunc_t)(int,const struct PDdata *);

void listens(objectfunc_t listener, char * name) {
  objectfunc_t *functions;
  ENTRY query, *returned;
  int i;

  query.key = name;
  query.data = NULL;

  returned = hsearch(query, FIND);
  if (returned == NULL) {
    functions = calloc(2,sizeof(objectfunc_t));
    if (functions == NULL) {
      error("insufficient memory for send/receive names table (%s %p)",name,listener);
      exit(1); /* should this be fatal? */
    } else {
      functions[0] = listener;
    }
    query.data = functions;
    returned = hsearch(query, ENTER);
    if (returned == NULL) {
      error("Exceeded number of send/receive names allowed");
    }
  } else {
    functions = returned->data;
    for (i=0 ; functions[i]!=NULL ; i++);
    fprintf(stderr,"[listens] Found %d elts, becoming %d bytes big-\n",i,(i+2)*sizeof(objectfunc_t));
    functions = realloc(functions,(i+2)*sizeof(objectfunc_t)); //2 b/c we don't include the NULL end-of-chain marker
    if (functions == NULL) {
      error("insufficient memory for send/receive names table (%s %p)",name,listener);
      exit(1); /* should this be fatal? */
    }
    functions[i] = listener; functions[i+1] = NULL;
    returned->data = functions;
  }

}

void sendbyname(char * name, const struct PDdata * message) {
  objectfunc_t *functions;
  ENTRY query, *returned;
  
  query.key = name;
  returned = hsearch(query,FIND);
  if (returned == NULL) {
    error("No-one listening to message sent to %s",name);
  } else {
    for (functions = returned->data;
	 *functions != NULL;
	 functions++)
      (*functions)(PDC_SENT,message);
  }
}

int linkedlength(const struct PDdata * m) {
  const struct PDdata * stepalong;
  int i;
 
  for (i=0,stepalong=m;stepalong!=NULL;i++,stepalong=stepalong->linked);
  return i;
}

const struct PDdata * linkedelt(const struct PDdata * m, int n) {
  const struct PDdata * i;

  for (i=m;n && i;n--,i=i->linked);
  return i;
}

#endif /* __PUDAC_H__ */
