# -*- cperl -*-

#  This file is part of PuDaC.

#  PuDaC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.

#  PuDaC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with PuDaC  If not, see <http://www.gnu.org/licenses/>.

%parser =
  ('#A' => sub {
     local *__ANON__ = 'PuDaC::Parser::A';
     $previousarray->{'contents'} = \@_;
     # "#A" contains initial data for the preceeding "#X array" object
     # for this, i need to keep a pointer to it lying around

     # The presence of a #A seems to ignore whether the previous object
     # nominally saved its contents.
   },
   '#N' => sub {
     local *__ANON__ = 'PuDaC::Parser::N';
     # currently there are only "#N canvas"es
     croak "Do not understand new $_[0]"
       unless (lc $_[0] eq "canvas");
     # this creates a canvas, used for patches, subpatches and graphs
     #  I don't want to handle it yet.
     # As a workaround for not having graphs, use [table arrayname]
     # Because they can be nested, each time we find one we need to
     #  ignore another depth until we find another "#X restore"

     # three kinds of #N canvas:
     #   #N canvas [X] [Y] [W] [H] [PtSz]   = First line only, patcher
     #   #N canvas [X] [Y] [W] [H] [name] [OpenOnLoad] = subpatcher or graph

     #  name is something like "(subpatch)" in the case of unnamed subpatchers
     #   or graphs
     #  name does not need to be unique

     # possible working suggestion: each subpatcher's objects start counting at
     #  1k higher than the last one; PD deals sufficiently poorly with 1k objects
     #  that I doubt anyone will have a patch with 1k objects in it.

     # If I'm wrong, I can just increase it to 10k at not worry about it.

     # The real problem comes from keeping track of multiple subpatches --
     #  i guess we need a stack

     $number_of_canvases++;
     if ($number_of_canvases > 1) {
       croak "We don't yet deal with subpatches, sorry.\n".
	 "If you need arrays, try using the [table] object";
     }
   },
   '#X' => sub {
     local *__ANON__ = 'PuDaC::Parser::X';
     my ($name, @rem) = @_;
     if (exists $xtypes{$name}) {
       $xtypes{$name}->(@rem)
     } else {
       croak "Don't undertstand #X $name";
     }
     # All X types increase the object count, even comments.
     #    EXCEPT NOT CONNECTs
     #    (coords probably don't either)
     #    but these both tend to be the last items in the list
     #    and so don't matter for numbering
     # So the patcher containing
     #   [X] 1
     #  4 |    comment 2
     #   [X] 3
     # has the connect going from object 0 to object 2
     }
   );

1;
