# -*- cperl -*-

#  This file is part of PuDaC.

#  PuDaC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.

#  PuDaC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with PuDaC  If not, see <http://www.gnu.org/licenses/>.



%xtypes =
  ('array' => sub {
     local *__ANON__ = 'PuDaC::Xtypes::array';
     # #X array [name] [sz] float [Properties];
     # #X obj [X] [Y] table [name] [sz];
     my ($name,$size,$check,$properties,@verify)=@_;
     carp "Array (object #$objectcount) is not float? (claims $check)"
       unless (lc $check eq "float");
     carp "Array (object #$objectcount) is unknown visual representation"
       unless (0 <= $properties and $properties <= 5);
     carp "Array (object #$objectcount) has extra arguments?"
       if ($#verify > 0);

     # properties:
     #  0- do not save, polyline
     #  1- save contents, polyline
     #  2- do not save, points
     #  3- save contents, points
     #  4- do not save, bezier
     #  5- save contents, bezier

     # Note that array is a kind of sugar for a "#X obj table"
     # they just always plot on parent (in fact, _all_ of the parent)
     # which we don't care about, because we're not doing graphics
     $objectlist{$objectcount}->{'CODE'} =
       $objecttranslations{'table'}->{'CODE'};
     $objectlist{$objectcount}->{'ARGS'} = [$name,$size];
     $previousarray = $objectlist{$objectcount};
     $objectcount++;
     # this is probably incomplete, i need to figure out how tabread/&c work
     },

   'connect' => sub {
     local *__ANON__ = 'PuDaC::Xtypes::connect';
     # #X connect object-out outlet# object-in inlet#

     # we build the $attachedobjectcalls strings here
     my ($objfrom, $outletnum, $objto, $inletnum,@verify) = @_;
     carp "Connect has extra arguments?"
       if ($#verify > 0);

     # Note: Connecting to non-existant outlets causes C compile time errors,
     #  while connecting to non-existant inlets causes runtime errors
     # (for comparison, invalid arguments should cause PUDAC compile tile errors)

     $objectlist{$objfrom}->{'AOCS'} .=
       "object${objto}($inletnum,&outvalue".
	 (($outletnum!=0)?($outletnum+1):"").
	   ");\n";

     # actually, other direction for DSP objects.

     # (This is the reverse-linked for use by DSP objects:)

     $objectlist{$objto}->{'DSPCS'} .=
       "object${objfrom}(".-($outletnum+1).",&DSPtrace".
	 ($inletnum+1).");\n";
     },

   'coords' => sub {
     local *__ANON__ = 'PuDaC::Xtypes::coords';
     # Coords seems to be purely visual?
     # I think we can ignore it.
     # It also doesn't count towards the object count.
     # (It also is written by PD as the last thing,
     #  so it oughtn't have mattered anyway)
   },

   'msg' => sub {
     local *__ANON__ = 'PuDaC::Xtypes::msg';
     my ($x,$y,@rest) = @_;
     # How do we do this? We probably make a special kind of object
     # that parses $0 and friends.

     # Messages by default send lists.
     # So    [ foo (    sends "foo"->NULL

     # Unlike many other objects, they keep no state
     # So    [ foo $1 (    sends "foo"->0->NULL
     #  unless it was sent something better than a bang

     # Msgs show one of the problems with the representation I've
     #  chosen for lists:
     # [ bob ( -- [ foo $1 (    errors and sends "foo"->0->NULL
     #  because it tries to interpret bob (an anything) as a float (why?)
     # However,
     # [ symbol bob ( -- [ foo $1 (   sends "foo"->"bob"->NULL
     #  because it's told to interpret bob as a symbol
     # Similarly,
     # [ list bob ( -- [ foo $1 (   sends "foo"->"bob"->NULL
     #  because ... the list ... you know, i have no idea.

     # Anyway, I'm just going to gloss over that, since I hope my compiler
     #  won't get me in trouble by being too tolerant.

     # The object name is named \MSG because Tk can't deal with backslashes, and
     #  and so PureData forbids them.
     $objectlist{$objectcount}->{'CODE'} =
       $objecttranslations{'\msg'}->{'CODE'};
     $objectlist{$objectcount}->{'ARGS'} = \@rest;
     $objectcount++;
   },

   'restore' => sub {
     local *__ANON__ = 'PuDaC::Xtypes::restore';
     my ($x, $y, $type, $name, @verify) = @_;

     carp "Restore has extra arguments?"
       if ($#verify > 0);

     croak "Don't yet deal with subpatches, sorry.\n";
   },
   # Atoms are interesting in that they're like a couple of objects together
   # (ignoring their graphic representation) :
   # * send, receive, [symbol/float]
   'floatatom' => sub {
     local *__ANON__ = 'PuDaC::Xtypes::floatatom';
     my ($x, $y, $width, $min, $max, $labelpos, $label, $rcv, $snd, @verify) = @_;

     carp "Floatatom has extra arguments?"
       if ($#verify > 0);

     # of the arguments, we don't care about any but rcv and snd,
     # because PureData doesn't clip inputs to floatatoms
     # this basically acts like a [float] object.

     push @{$receivers{$rcv}},$objectcount;

     $objectlist{$objectcount}->{'CODE'} =
       $objecttranslations{'\floatatom'}->{'CODE'};
     $objectlist{$objectcount}->{'ARGS'} = [$rcv,$snd];
     # we don't care about the other arguments because:
     # $x, $y, $width: only affect graphical appearance (or, when width=1, user input)
     # $min, $max: only affect user input (a floatatom can have an arbitrary # sent to it)
     # $labelpos, $label: only graphical

     # Sends tied to are processed after propagation
     #  there is a reason [send] objects don't have outlets

     $objectcount++;
   },
   # As far as I know, floatatom and symbolatom only accept the
   'symbolatom' => sub {
     local *__ANON__ = 'PuDaC::Xtypes::symbolatom';
     my ($x, $y, $width, $min, $max, $labelpos, $label, $rcv, $snd, @verify) = @_;

     carp "Symbolatom has extra arguments?"
       if ($#verify > 0);

     $objectcount++;

     croak "Symbol boxes unimplemented, sorry.\n";
   },
   # The obj's that have graphical representations have many of the same
   # properties as the atoms
   'obj' => sub {
     local *__ANON__ = 'PuDaC::Xtypes::obj';
     my ($x, $y, $name, @args) = @_;

     unless (exists $objecttranslations{$name}) {
       push @unimplementedObjects,$name;
       return;
     }

     $name = $objecttranslations{$name}->{'SYNONYM'}
       while (exists $objecttranslations{$name}->{'SYNONYM'});

     $objectlist{$objectcount}->{'CODE'} =
       $objecttranslations{$name}->{'CODE'};
     $objectlist{$objectcount}->{'ARGS'} = \@args;
     $objectcount++;
   },
   'text' => sub {
     local *__ANON__ = 'PuDaC::Xtypes::text';
     # These are comments. They get ignored.
     $objectcount++;
     # except they count as objects.
   },
  );

1;
